import asyncio
import time
import traceback
from enum import Enum
from .log import Log

class DI_config:
    def __init__(self, port, interval, edge_type):
        self._port = port
        self._interval = interval
        edge = EDGE.NONE
        if edge_type == "falling":
            edge = EDGE.FALL
        elif edge_type == "rising":
            edge = EDGE.RISE
        elif edge_type == "both":
            edge = EDGE.BOTH
        self._edge_type = edge
        self._state = 0
        self._start_time = time.time()
        self._edge_flag = EDGE.NONE
        self._notifi_edge = EDGE.NONE


class DO_config:
    def __init__(self,
                 start,
                 port,
                 output_state_isHigh,
                 output_time,
                 output_delay_time,
                 ):

        self._start = start
        self._state = STATE.NONE
        self._start_time = time.time()
        self._port = port
        self._output_state_isHigh = output_state_isHigh
        self._output_time = output_time
        self._output_delay_time = output_delay_time


class EDGE(Enum):
    NONE = 0
    FALL = 1
    RISE = 2
    BOTH = 3


class STATE(Enum):
    NONE = 0
    START = 1
    OUTPUT = 2
    END = 3


class SensingMgr:
    async def __async_loop(self, timeout=None):
        self.__curr_task = asyncio.current_task()

        while not self.__quit_requested:

            for conf in self._di_conf:
                curr_state = self._dio_mgr.get_DI(conf._port)

                # check edge
                if conf._state is not curr_state:
                    if conf._state is True:
                        conf._edge_flag = EDGE.FALL
                    else:
                        conf._edge_flag = EDGE.RISE

                if conf._notifi_edge is EDGE.NONE:
                    conf._notifi_edge = conf._edge_flag

                # set edge
                if (time.time() - conf._start_time) > conf._interval:
                    conf._start_time = time.time()
                    if conf._notifi_edge is EDGE.FALL and \
                            conf._edge_type is EDGE.FALL:
                        self._dio_mgr.set_DI_edge(conf._port, 1)
                    elif conf._notifi_edge is EDGE.RISE and \
                            conf._edge_type is EDGE.RISE:
                        self._dio_mgr.set_DI_edge(conf._port, 1)
                    elif conf._notifi_edge is not EDGE.NONE and \
                            conf._edge_type is EDGE.BOTH:
                        self._dio_mgr.set_DI_edge(conf._port, 1)
                    else:
                        self._dio_mgr.set_DI_edge(conf._port, 0)
                    conf._notifi_edge = EDGE.NONE

            # DO
            for conf in self._do_conf:
                if conf._start is False:
                    break

                curr_time = time.time()
                elapsed_time = curr_time - conf._start_time

                if conf._state is STATE.NONE:
                    conf._start_time = curr_time
                    conf._state = STATE.START

                elif conf._state is STATE.START:
                    if elapsed_time > float(conf._output_delay_time):
                        self._dio_mgr.set_DO_Output(
                            conf._port, conf._output_state_isHigh)
                        conf._start_time = curr_time
                        conf._state = STATE.OUTPUT

                elif conf._state is STATE.OUTPUT:
                    if elapsed_time > float(conf._output_time):
                        self._dio_mgr.set_DO_Output(
                            conf._port, not conf._output_state_isHigh)
                        conf._state = STATE.END

            try:
                await asyncio.sleep(0.1)
            except asyncio.CancelledError:
                pass

    def start_loop(self, timeout=None):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            loop.run_until_complete(self.__async_loop(timeout))
        except Exception as e:
            log = Log.instance()
            Log.error(log, "sensing_manager error")
            Log.error(log, e.args)
            Log.error(log, traceback.format_exc())
            Log.preserved_error_log(log)
            pass

    def set_di_conf(self, port, interval, edge_type):
        for conf in self._di_conf:
            if port is conf._port:
                new_conf = conf
                new_conf._interval = interval
                new_conf._edge_type = edge_type
                new_conf._state = self._dio_mgr.get_DI(port)
                self._di_conf.remove(conf)
                self._di_conf.append(new_conf)

    def set_do_conf(
            self,
            port,
            output_state_isHigh,
            output_time,
            output_delay_time,
            ):

        for conf in self._do_conf:
            if port is conf._port:
                new_conf = conf
                new_conf._start = False
                new_conf._output_state_isHigh = output_state_isHigh
                new_conf._output_time = output_time
                new_conf._output_delay_time = output_delay_time

                self._do_conf.remove(conf)
                self._do_conf.append(new_conf)

    def start_do(self, port):
        for conf in self._do_conf:
            if port is conf._port:
                conf._start = True
                self._state = STATE.NONE
                self._start_time = time.time()

    def stop_do(self, port):
        for conf in self._do_conf:
            if port is conf._port:
                conf._start = False

    def request_stop(self):
        self.__quit_requested = True
        self.__curr_task.cancel()

    def create_di_conf(self, port, interval, edge_type):
        di_conf = DI_config(port, interval, edge_type)
        di_conf._state = self._dio_mgr.get_DI(port)
        self._di_conf.append(di_conf)

    def create_do_conf(
            self,
            port,
            start,
            output_state_isHigh,
            output_time,
            output_delay_time,
            ):
        do_conf = DO_config(start, port, output_state_isHigh, output_time,
                            output_delay_time
                            )
        self._do_conf.append(do_conf)

    def __init__(self, dio_mgr):
        self._dio_mgr = dio_mgr
        self._di_conf = []
        self._do_conf = []
        self.__quit_requested = False

#
# End of File
#
